package com.supervision.manage.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.exception.BusinessException;
import com.supervision.model.Disease;
import com.supervision.vo.manage.DiseaseVo;
import com.supervision.service.DiseaseService;
import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

@Api(tags = "疾病管理")
@RestController
@RequestMapping("disease")
@RequiredArgsConstructor
public class DiseaseController {

    private final DiseaseService diseaseService;


    @PostMapping("/save")
    @ApiOperation("新增疾病")
    @ApiModelProperty
    public Disease saveDisease(@RequestBody Disease disease) {

        return diseaseService.saveDisease(disease);
    }

    @ApiOperation("删除疾病")
    @DeleteMapping("/delete")
    public boolean deleteDisease(@RequestParam("疾病id") String id) {

        if (StrUtil.isEmpty(id)){
            throw new BusinessException("id is not allow empty");
        }
        return diseaseService.removeById(id);

    }


    @ApiOperation("修改辅助检查信息")
    @PutMapping("/update")
    public boolean updateDisease(Disease disease) {

        return diseaseService.updateDisease(disease);

    }


    @ApiOperation("查询疾病列表")
    @GetMapping("/queryPageList")
    public Page<Disease> queryPageList(@RequestParam(defaultValue = "1")Integer pageNum,
                                       @RequestParam(defaultValue = "10")Integer pageSize,
                                       @RequestParam DiseaseVo diseaseVo) {

        return diseaseService.queryPageList(pageNum,pageSize,diseaseVo);

    }








}
