package com.supervision.manage.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.exception.BusinessException;
import com.supervision.manage.pojo.vo.MedicalRecManageVO;
import com.supervision.model.AskPatientAnswer;
import com.supervision.model.MedicalRec;
import com.supervision.model.Patient;
import com.supervision.service.AskPatientAnswerService;
import com.supervision.vo.manage.MedicalRecPageResVO;
import com.supervision.manage.service.MedicalRecManageService;
import com.supervision.model.Disease;
import com.supervision.service.DiseaseService;
import com.supervision.service.MedicalRecService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class MedicalRecManageServiceImpl implements MedicalRecManageService {

    private final DiseaseService diseaseService;

    private final MedicalRecService medicalRecService;

    private final AskPatientAnswerService askPatientAnswerService;

    public List<Disease> queryDiseaseListByKeyWord(String keyword) {
        // 注意,这里不支持查询复杂疾病,如果需要支持复杂疾病,这里需要单独进行改造
        return diseaseService.lambdaQuery().eq(Disease::getDiseaseType, 0).like(Disease::getDiseaseName, keyword).list();
    }

    @Override
    public IPage<MedicalRecPageResVO> queryMedicalRecManagePage(String selfDescKeyword, String gender, String diseaseId, Integer pageNum, Integer pageSize) {
        // 注意,这里的diseaseId不支持查询复杂疾病,如果需要支持复杂疾病,这里需要单独进行改造
        return medicalRecService.queryMedicalRecManagePage(selfDescKeyword, gender, diseaseId, pageNum, pageSize);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createMedicalRec(MedicalRecManageVO reqVO) {
        // 首先新增病人
        Patient patient = reqVO.getPatient();
        patient.insert();
        // 然后保存病历
        MedicalRec medicalRec = BeanUtil.toBean(reqVO, MedicalRec.class);
        medicalRec.setPatientId(patient.getId());
        // 注意,这里新增的时候,目前只支持单个疾病,如果需要多个疾病,需要在这里改造,先新增出来一个疾病
        medicalRec.setDiseaseId(reqVO.getDiseaseId());
        medicalRec.insert();
        // 这里生成一个新的编号(首先去数据库里面查询编号,然后用最大的编号+1)(可能存在并发修改导致编码重复问题,不是重点,等有问题再说)
        medicalRecService.updateMedicalRecNo("QL", "男".equals(patient.getGender()) ? "M" : "F", medicalRec.getId());
        // 然后保存病历编码
        List<AskPatientAnswer> qaList = reqVO.getQaList();
        for (AskPatientAnswer askPatientAnswer : qaList) {
            if (StrUtil.isBlank(askPatientAnswer.getLibraryQuestionId())) {
                throw new BusinessException("问题ID不能为空");
            }
            askPatientAnswer.setMedicalId(medicalRec.getId());
            askPatientAnswer.setPatientId(patient.getId());
        }
        askPatientAnswerService.saveBatch(qaList);
    }
}
