package com.supervision.model;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.supervision.handler.StringListTypeHandler;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.io.Serializable;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 诊断进程表
 * @TableName vp_rasa_model_info
 */
@TableName(value ="vp_rasa_model_info" ,autoResultMap = true)
@Data
@ApiModel
public class RasaModelInfo implements Serializable {

    /**
     * 主键
     */
    @TableId
    private String id;

    /**
     * 模型ID 与vp_disease的主键进行关联
     */
    @ApiModelProperty("模型ID")
    private String modelId;

    /**
     * 中文注释
     */
    @ApiModelProperty("中文注释")
    private String description;

    /**
     * 模型对应的端口号
     */
    @ApiModelProperty("模型对应的端口号")
    private Integer port;


    /**
     * 训练状态 0:训练失败 1:训练成功
     */
    @ApiModelProperty("训练状态")
    private Integer tranStatus;

    /**
     * 启动状态 -1:未启动 0:启动失败 1:启动成功
     */
    @ApiModelProperty("启动状态")
    private Integer serverStatus;


    @ApiModelProperty("执行train shell命令")
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> trainCmd;


    @ApiModelProperty("执行run shell命令")
    @TableField(typeHandler = StringListTypeHandler.class)
    private List<String> runCmd;


    @ApiModelProperty("训练日志")
    private String trainLog;

    @ApiModelProperty("运行日志")
    private String runLog;


    /**
     * 创建人ID
     */
    private String createUserId;

    /**
     * 创建时间
     */
    private LocalDateTime createTime;

    /**
     * 更新人
     */
    private String updateUserId;

    /**
     * 更新时间
     */
    private LocalDateTime updateTime;

    @TableField(exist = false)
    private static final long serialVersionUID = 1L;

}
