package com.supervision.rasa.config;

import java.util.concurrent.*;

public class ThreadPoolExecutorConfig {

    private volatile static ThreadPoolExecutor instance = null;

    private ThreadPoolExecutorConfig(){}


    public static ThreadPoolExecutor getInstance() {
        if (instance == null) {
            synchronized (ThreadPoolExecutorConfig.class) {  // 加锁
                if (instance == null) {
                    int corePoolSize = 5;
                    int maximumPoolSize = 10;
                    long keepAliveTime = 100;
                    BlockingQueue<Runnable> workQueue = new ArrayBlockingQueue<>(20);
                    RejectedExecutionHandler rejectedExecutionHandler = new ThreadPoolExecutor.AbortPolicy();
                    instance = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime, TimeUnit.SECONDS, workQueue, rejectedExecutionHandler);
                }
            }
        }
        return instance;
    }

}
