package com.supervision.rasa.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.supervision.rasa.service.RasaFileService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;

@Service
@Slf4j
@RequiredArgsConstructor
public class RasaFileServiceImpl implements RasaFileService {


    @Value("${rasa.data-path:/home/rasa/model_resource/}")
    private String rasaFilePath;

    @Value("${rasa.file-name:rasa.zip}")
    private String rasaFileName;


    @Override
    public void saveRasaFile(MultipartFile file,String modelId) throws IOException {


        String suffix = "_back";
        String rasaFullPath = String.join(File.separator, rasaFilePath,modelId, rasaFileName);
        String rasaBackFullPath = rasaFullPath + suffix;

        //初始化目录
        File dir = new File(String.join(File.separator, rasaFilePath,modelId));
        if (!dir.exists()){
            FileUtil.mkdir(dir);
        }

        //1.检查路径下是否存在文件
        File oldFile = new File(rasaFullPath);
        if (oldFile.exists()){
            //1.1 如果存在文件，先备份文件
            FileUtil.rename(oldFile,rasaBackFullPath,true);
        }


        try {
            //2.把流文件保存到本地
            file.transferTo(new File(rasaFullPath));

            //3.解压文件
            ZipUtil.unzip(rasaFullPath,String.join(File.separator, rasaFilePath,modelId));

            //4.删除备份文件
            FileUtil.del(rasaBackFullPath);
        } catch (IOException e) {
            // 恢复文件
            File backFile = new File(rasaBackFullPath);
            if (backFile.exists()){ //恢复文件
                FileUtil.rename(backFile,rasaFileName,true);
            }
            throw new IOException(e);
        }
    }

}
